package postservicelogic

import (
	"app-cloudep-tweeting-service/internal/domain"
	"context"

	"app-cloudep-tweeting-service/gen_result/pb/tweeting"
	"app-cloudep-tweeting-service/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type DeletePostLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewDeletePostLogic(ctx context.Context, svcCtx *svc.ServiceContext) *DeletePostLogic {
	return &DeletePostLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// DeletePost 刪除貼文
func (l *DeletePostLogic) DeletePost(in *tweeting.DeletePostsReq) (*tweeting.OKResp, error) {
	_, err := l.svcCtx.PostModel.DeleteMany(l.ctx, in.GetPostId()...)
	if err != nil {
		e := domain.PostMongoErrorL(
			logx.WithContext(l.ctx),
			[]logx.LogField{
				{Key: "req", Value: in},
				{Key: "func", Value: "PostModel.DeletePost"},
				{Key: "err", Value: err},
			},
			"failed to add del post").Wrap(err)
		return nil, e
	}

	return &tweeting.OKResp{}, nil
}
