package postservicelogic

import (
	"app-cloudep-tweeting-service/internal/domain"
	model "app-cloudep-tweeting-service/internal/model/mongo"
	"context"

	ers "code.30cm.net/digimon/library-go/errs"
	"go.mongodb.org/mongo-driver/bson/primitive"

	"app-cloudep-tweeting-service/gen_result/pb/tweeting"
	"app-cloudep-tweeting-service/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type UpdatePostLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewUpdatePostLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UpdatePostLogic {
	return &UpdatePostLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

type checkPostId struct {
	PostID  string `validate:"required"`
	Content string `json:"content,omitempty" validate:"lte=500"`
}

// UpdatePost 更新貼文
func (l *UpdatePostLogic) UpdatePost(in *tweeting.UpdatePostReq) (*tweeting.OKResp, error) {
	// 驗證資料
	if err := l.svcCtx.Validate.ValidateAll(&checkPostId{
		PostID:  in.GetPostId(),
		Content: in.GetContent(),
	}); err != nil {
		return nil, ers.InvalidFormat(err.Error())
	}
	// 沒有就沒有，有就走全覆蓋
	update := model.Post{}
	oid, err := primitive.ObjectIDFromHex(in.GetPostId())
	if err != nil {
		return nil, ers.InvalidFormat("failed to get correct post id")
	}
	update.ID = oid
	update.Tags = in.GetTags()
	// 將 Media 存入
	var media []model.Media
	for _, item := range in.GetMedia() {
		media = append(media, model.Media{
			Links: item.Url,
			Type:  item.Type,
		})
	}
	update.Media = media
	update.Content = in.GetContent()
	update.Like = uint64(in.GetLikeCount())
	update.DisLike = uint64(in.GetDislikeCount())

	_, err = l.svcCtx.PostModel.UpdateOptional(l.ctx, &update)
	if err != nil {
		e := domain.PostMongoErrorL(
			logx.WithContext(l.ctx),
			[]logx.LogField{
				{Key: "req", Value: in},
				{Key: "func", Value: "PostModel.UpdateOptional"},
				{Key: "err", Value: err},
			},
			"failed to add new post").Wrap(err)
		return nil, e
	}

	return &tweeting.OKResp{}, nil
}
